"""
Bulk walk MIB
+++++++++++++

Send a series of SNMP GETBULK requests using the following options:

* with SNMPv2c, community 'public'
* over IPv4/UDP
* to an Agent at demo.pysnmp.com:161
* for all OIDs past SNMPv2-MIB::system
* run till end-of-mib condition is reported by Agent
* based on asyncio I/O framework

Functionally similar to:

| $ snmpbulkwalk -v2c -c public -Cn0 -Cr50 \
|                demo.pysnmp.com  SNMPv2-MIB::system

"""  #

import asyncio
from pysnmp.hlapi.v1arch.asyncio import *


async def run(varBinds):
    with SnmpDispatcher() as snmpDispatcher:
        while True:
            iterator = await bulk_cmd(
                snmpDispatcher,
                CommunityData("public"),
                await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
                0,
                50,
                *varBinds,
            )

            errorIndication, errorStatus, errorIndex, varBindTable = iterator

            if errorIndication:
                print(errorIndication)
                break

            elif errorStatus:
                print(
                    "{} at {}".format(
                        errorStatus.prettyPrint(),
                        errorIndex and varBinds[int(errorIndex) - 1][0] or "?",
                    )
                )
            else:
                for varBind in varBindTable:
                    print(" = ".join([x.prettyPrint() for x in varBind]))

            varBinds = varBindTable
            if is_end_of_mib(varBinds):
                break


asyncio.run(run([ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr"))]))



"""
SNMPv1
++++++

Send SNMP GET request using the following options:

  * with SNMPv1, community 'public'
  * over IPv4/UDP
  * to an Agent at demo.pysnmp.com:161
  * for the 1.3.6.1.2.1.1.1.0 OID (e.g. SNMPv2-MIB::sysDescr.0 MIB object)
  * Based on asyncio I/O framework

Functionally similar to:

| $ snmpget -v1 -c public demo.pysnmp.com SNMPv2-MIB::sysDescr.0

"""  #

import asyncio
from pysnmp.hlapi.v1arch.asyncio import *


async def run():
    with SnmpDispatcher() as snmpDispatcher:
        iterator = await get_cmd(
            snmpDispatcher,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
            ("1.3.6.1.2.1.1.1.0", None),
        )

        errorIndication, errorStatus, errorIndex, varBinds = iterator

        if errorIndication:
            print(errorIndication)

        elif errorStatus:
            print(
                "{} at {}".format(
                    errorStatus.prettyPrint(),
                    errorIndex and varBinds[int(errorIndex) - 1][0] or "?",
                )
            )
        else:
            for varBind in varBinds:
                print(" = ".join([x.prettyPrint() for x in varBind]))


asyncio.run(run())



import math
import pytest

from pysnmp.hlapi.v3arch.asyncio import *
from tests.agent_context import AGENT_PORT, AgentContextManager

total_count = 68  # Ensure this matches the actual total count of SNMP objects


@pytest.mark.asyncio
@pytest.mark.parametrize("max_repetitions", [1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30])
async def test_v2c_get_table_bulk(max_repetitions):
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            objects = bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                # await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
                ContextData(),
                0,
                max_repetitions,
                ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
            )

            objects_list = [obj async for obj in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == max_repetitions
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == max_repetitions
            # assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysName.0"

            assert len(objects_list) == math.ceil(total_count / max_repetitions)


@pytest.mark.asyncio
async def test_v2c_get_table_bulk_0_4_subtree():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            index = 0
            async for (
                errorIndication,
                errorStatus,
                errorIndex,
                varBinds,
            ) in bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ContextData(),
                0,
                4,
                ObjectType(ObjectIdentity("SNMPv2-MIB", "snmp")),
                lexicographicMode=False,
            ):
                assert errorIndication is None
                assert errorStatus == 0
                assert len(varBinds) == 4
                if index == 0:
                    assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::snmpInPkts.0"

                if index == 1:
                    assert (
                        varBinds[0][0].prettyPrint()
                        == "SNMPv2-MIB::snmpInBadCommunityUses.0"
                    )

                if index == 26:
                    assert (
                        varBinds[0][0].prettyPrint() == "SNMPv2-MIB::snmpSilentDrops.0"
                    )

                if index == 27:
                    assert (
                        varBinds[0][0].prettyPrint() == "SNMPv2-MIB::snmpProxyDrops.0"
                    )

                index += 1

            assert index == 7


@pytest.mark.asyncio
async def test_v2c_get_table_bulk_0_1_subtree():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            index = 0
            async for (
                errorIndication,
                errorStatus,
                errorIndex,
                varBinds,
            ) in bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ContextData(),
                0,
                1,
                ObjectType(ObjectIdentity("SNMPv2-MIB", "snmp")),
                lexicographicMode=False,
            ):
                assert errorIndication is None
                assert errorStatus == 0
                assert len(varBinds) == 1
                if index == 0:
                    assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::snmpInPkts.0"

                if index == 1:
                    assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::snmpOutPkts.0"

                if index == 26:
                    assert (
                        varBinds[0][0].prettyPrint() == "SNMPv2-MIB::snmpSilentDrops.0"
                    )

                if index == 27:
                    assert (
                        varBinds[0][0].prettyPrint() == "SNMPv2-MIB::snmpProxyDrops.0"
                    )

                index += 1

            assert index == 28


@pytest.mark.asyncio
async def test_v2c_get_table_bulk_0_7():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            max_repetitions = 7
            objects = bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                # await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
                ContextData(),
                0,
                max_repetitions,
                ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
            )

            objects_list = [obj async for obj in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == max_repetitions
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == max_repetitions
            # assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysName.0"

            assert len(objects_list) == math.ceil(total_count / max_repetitions)


@pytest.mark.asyncio
async def test_v2c_get_table_bulk_0_8():
    with SnmpEngine() as snmpEngine:
        max_repetitions = 8
        objects = bulk_walk_cmd(
            snmpEngine,
            CommunityData("public"),
            # await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
            ContextData(),
            0,
            max_repetitions,
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
        )

        objects_list = [obj async for obj in objects]

        errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == max_repetitions
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

        errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == max_repetitions
        # assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysName.0"

        assert len(objects_list) == math.ceil(total_count / max_repetitions)


@pytest.mark.asyncio
async def test_v2c_get_table_bulk_0_35():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            max_repetitions = 35  # (68/2) + 1
            objects = bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                # await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
                ContextData(),
                0,
                max_repetitions,
                ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
            )

            objects_list = [obj async for obj in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == max_repetitions
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == total_count - max_repetitions
            # assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysName.0"

            assert len(objects_list) == math.ceil(total_count / max_repetitions)


@pytest.mark.asyncio
async def test_v2c_get_table_bulk_0_60():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            max_repetitions = 60
            objects = bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                # await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
                ContextData(),
                0,
                max_repetitions,
                ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
            )

            objects_list = [obj async for obj in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == max_repetitions
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

            assert len(objects_list) == math.ceil(total_count / max_repetitions)


@pytest.mark.asyncio
async def test_v2c_get_table_bulk_0_5_subtree():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            max_repetitions = 5
            objects = bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                # await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
                ContextData(),
                0,
                max_repetitions,
                ObjectType(ObjectIdentity("SNMPv2-MIB", "system")),
                lexicographicMode=False,
            )

            objects_list = [obj async for obj in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == max_repetitions
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[3]
            assert len(varBinds) == 1

            assert len(objects_list) == 4


@pytest.mark.asyncio
async def test_v2c_get_table_bulk_0_6_subtree():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            max_repetitions = 6
            objects = bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                # await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
                ContextData(),
                0,
                max_repetitions,
                ObjectType(ObjectIdentity("SNMPv2-MIB", "system")),
                lexicographicMode=False,
            )

            objects_list = [obj async for obj in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == max_repetitions
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[2]
            assert len(varBinds) == 4

            assert len(objects_list) == 3


@pytest.mark.asyncio
async def test_bulk_walk_lookupmib_true():
    """
    Test that bulk_walk_cmd() works correctly with lookupMib=True.
    This should always work, even before the bug fix.
    """
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            count = 0
            objects = bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ContextData(),
                0,
                1,  # Small max_repetitions to ensure multiple queries
                ObjectType(ObjectIdentity("SNMPv2-MIB", "system")),
                lookupMib=True,
            )

            # This should work fine
            async for errorIndication, errorStatus, errorIndex, varBinds in objects:
                assert errorIndication is None, (
                    f"Error with lookupMib=True: {errorIndication}"
                )
                count += 1
                if count >= 5:  # Process at least some responses
                    break

            assert count > 0, "No lookupMib=True responses processed"


@pytest.mark.asyncio
async def test_bulk_walk_lookupmib_false():
    """
    Test that bulk_walk_cmd() works correctly with lookupMib=False.

    Previously this would fail because when lookupMib=False, the unmake_varbinds() helper
    function would return plain ObjectName objects instead of ObjectIdentity objects,
    which would cause a type mismatch when creating the next query.

    Now it works properly even with multiple queries due to the fix in bulk_walk_cmd().
    """
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            count = 0

            objects = bulk_walk_cmd(
                snmpEngine,
                CommunityData("public"),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ContextData(),
                0,
                1,  # Small max_repetitions to ensure multiple queries
                ObjectType(ObjectIdentity("SNMPv2-MIB", "system")),
                lookupMib=False,
            )

            # With the bug fixed, this should now work correctly
            async for errorIndication, errorStatus, errorIndex, varBinds in objects:
                assert errorIndication is None, (
                    f"Error with lookupMib=False: {errorIndication}"
                )
                count += 1
                if count >= 5:  # Process at least some responses to confirm it works
                    break

            # We should be able to process multiple responses without error now
            assert count > 0, "No responses processed with lookupMib=False"
            assert count >= 2, "Expected multiple responses with lookupMib=False"



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
import random
from hashlib import md5, sha1

from pyasn1.type import univ
from pysnmp.proto import errind, error
from pysnmp.proto.secmod.rfc3414 import localkey
from pysnmp.proto.secmod.rfc3414.auth import hmacmd5, hmacsha
from pysnmp.proto.secmod.rfc3414.priv import base
from pysnmp.proto.secmod.rfc7860.auth import hmacsha2

PysnmpCryptoError = False
try:
    from cryptography.hazmat.backends import default_backend
    from cryptography.hazmat.primitives.ciphers import algorithms, Cipher, modes

except ImportError:
    PysnmpCryptoError = True


random.seed()


# RFC3826

#


class Aes(base.AbstractEncryptionService):
    """AES encryption protocol.

    https://tools.ietf.org/html/rfc3826
    """

    SERVICE_ID = (1, 3, 6, 1, 6, 3, 10, 1, 2, 4)  # usmAesCfb128Protocol
    KEY_SIZE = 16
    local_int = random.randrange(0, 0xFFFFFFFFFFFFFFFF)

    # 3.1.2.1
    def __get_encryption_key(self, privKey, snmpEngineBoots, snmpEngineTime):
        salt = [
            self.local_int >> 56 & 0xFF,
            self.local_int >> 48 & 0xFF,
            self.local_int >> 40 & 0xFF,
            self.local_int >> 32 & 0xFF,
            self.local_int >> 24 & 0xFF,
            self.local_int >> 16 & 0xFF,
            self.local_int >> 8 & 0xFF,
            self.local_int & 0xFF,
        ]

        if self.local_int == 0xFFFFFFFFFFFFFFFF:
            self.local_int = 0
        else:
            self.local_int += 1

        return self.__get_decryption_key(
            privKey, snmpEngineBoots, snmpEngineTime, salt
        ) + (univ.OctetString(salt).asOctets(),)

    def __get_decryption_key(self, privKey, snmpEngineBoots, snmpEngineTime, salt):
        snmpEngineBoots, snmpEngineTime, salt = (
            int(snmpEngineBoots),
            int(snmpEngineTime),
            salt,
        )

        iv = [
            snmpEngineBoots >> 24 & 0xFF,
            snmpEngineBoots >> 16 & 0xFF,
            snmpEngineBoots >> 8 & 0xFF,
            snmpEngineBoots & 0xFF,
            snmpEngineTime >> 24 & 0xFF,
            snmpEngineTime >> 16 & 0xFF,
            snmpEngineTime >> 8 & 0xFF,
            snmpEngineTime & 0xFF,
        ] + salt

        return privKey[: self.KEY_SIZE].asOctets(), univ.OctetString(iv).asOctets()

    def hash_passphrase(self, authProtocol, privKey) -> univ.OctetString:
        """Hash a passphrase."""
        if authProtocol == hmacmd5.HmacMd5.SERVICE_ID:
            hashAlgo = md5
        elif authProtocol == hmacsha.HmacSha.SERVICE_ID:
            hashAlgo = sha1
        elif authProtocol in hmacsha2.HmacSha2.HASH_ALGORITHM:
            hashAlgo = hmacsha2.HmacSha2.HASH_ALGORITHM[authProtocol]
        else:
            raise error.ProtocolError(f"Unknown auth protocol {authProtocol}")
        return localkey.hash_passphrase(privKey, hashAlgo)

    def localize_key(self, authProtocol, privKey, snmpEngineID) -> univ.OctetString:
        """Localize a key."""
        if authProtocol == hmacmd5.HmacMd5.SERVICE_ID:
            hashAlgo = md5
        elif authProtocol == hmacsha.HmacSha.SERVICE_ID:
            hashAlgo = sha1
        elif authProtocol in hmacsha2.HmacSha2.HASH_ALGORITHM:
            hashAlgo = hmacsha2.HmacSha2.HASH_ALGORITHM[authProtocol]
        else:
            raise error.ProtocolError(f"Unknown auth protocol {authProtocol}")
        localPrivKey = localkey.localize_key(privKey, snmpEngineID, hashAlgo)
        return localPrivKey[: self.KEY_SIZE]

    # 3.2.4.1
    def encrypt_data(self, encryptKey, privParameters, dataToEncrypt):
        """Encrypt data."""
        if PysnmpCryptoError:
            raise error.StatusInformation(errorIndication=errind.encryptionError)

        snmpEngineBoots, snmpEngineTime, salt = privParameters

        # 3.3.1.1
        aesKey, iv, salt = self.__get_encryption_key(
            encryptKey, snmpEngineBoots, snmpEngineTime
        )

        # 3.3.1.3
        try:
            aes = _cryptography_cipher(aesKey, iv).encryptor()
            ciphertext = aes.update(dataToEncrypt) + aes.finalize()

        except AttributeError:
            raise error.StatusInformation(
                errorIndication=errind.unsupportedPrivProtocol
            )

        # 3.3.1.4
        return univ.OctetString(ciphertext), univ.OctetString(salt)

    # 3.2.4.2
    def decrypt_data(self, decryptKey, privParameters, encryptedData):
        """Decrypt data."""
        if PysnmpCryptoError:
            raise error.StatusInformation(errorIndication=errind.decryptionError)

        snmpEngineBoots, snmpEngineTime, salt = privParameters

        # 3.3.2.1
        if len(salt) != 8:
            raise error.StatusInformation(errorIndication=errind.decryptionError)

        # 3.3.2.3
        aesKey, iv = self.__get_decryption_key(
            decryptKey, snmpEngineBoots, snmpEngineTime, salt
        )

        try:
            # 3.3.2.4-6
            aes = _cryptography_cipher(aesKey, iv).decryptor()
            return aes.update(encryptedData.asOctets()) + aes.finalize()

        except AttributeError:
            raise error.StatusInformation(
                errorIndication=errind.unsupportedPrivProtocol
            )


def _cryptography_cipher(key, iv):
    """Build a cryptography AES Cipher object.

    :param bytes key: Encryption key
    :param bytes iv: Initialization vector
    :returns: AES Cipher instance
    :rtype: cryptography.hazmat.primitives.ciphers.Cipher
    """
    return Cipher(
        algorithm=algorithms.AES(key), mode=modes.CFB(iv), backend=default_backend()
    )



import pytest
from pysnmp.hlapi.v1arch.asyncio import *
from tests.agent_context import AGENT_PORT, AgentContextManager


@pytest.mark.asyncio
async def test_v2_get():
    async with AgentContextManager():
        with SnmpDispatcher() as snmpDispatcher:
            errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
                snmpDispatcher,
                CommunityData("public"),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
            )

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == 1
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
            assert varBinds[0][1].prettyPrint().startswith("PySNMP engine version")
            assert isinstance(varBinds[0][1], OctetString)


@pytest.mark.asyncio
async def test_v2_get_no_access_object():
    async with AgentContextManager(enable_custom_objects=True):
        with SnmpDispatcher() as snmpDispatcher:
            errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
                snmpDispatcher,
                CommunityData("public"),
                await UdpTransportTarget.create(
                    ("localhost", AGENT_PORT), timeout=1, retries=0
                ),
                ObjectType(ObjectIdentity("1.3.6.1.4.1.60069.9.3")),
            )

            assert errorIndication is None
            assert errorStatus.prettyPrint() == "noAccess"  # v2c and v3 use noAccess


@pytest.mark.asyncio
async def test_v2_get_legacy_object():
    async with AgentContextManager(enable_custom_objects=True):
        with SnmpDispatcher() as snmpDispatcher:
            errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
                snmpDispatcher,
                CommunityData("public"),
                await UdpTransportTarget.create(
                    ("localhost", AGENT_PORT), timeout=1, retries=0
                ),
                ObjectType(ObjectIdentity("1.3.6.1.4.1.60069.9.4")),
            )

            assert errorIndication is None
            assert (
                errorStatus.prettyPrint() == "noAccess"
            )  # PySMI <1.3.0 generates such objects



"""
SNMPv2c TRAP
++++++++++++

Send SNMP TRAP notification using the following options:

* SNMPv2c
* with community name 'public'
* over IPv4/UDP
* send TRAP notification
* to a Manager at 127.0.0.1:162
* with TRAP ID 'coldStart' specified as an OID
* include managed objects information:
  1.3.6.1.2.1.1.1.0 = 'Example Notificator'
  1.3.6.1.2.1.1.5.0 = 'Notificator Example'

Functionally similar to:

| $ snmptrap -v2c -c public 127.0.0.1 12345 1.3.6.1.4.1.20408.4.1.1.2

"""  #
from pysnmp.entity import engine, config
from pysnmp.carrier.asyncio.dgram import udp
from pysnmp.entity.rfc3413 import ntforg
from pysnmp.proto.api import v2c

# Create SNMP engine instance
snmpEngine = engine.SnmpEngine()

# SecurityName <-> CommunityName mapping
config.add_v1_system(snmpEngine, "my-area", "public", transportTag="all-my-managers")

# Specify security settings per SecurityName (SNMPv2c -> 1)
config.add_target_parameters(snmpEngine, "my-creds", "my-area", "noAuthNoPriv", 1)

# Setup transport endpoint and bind it with security settings yielding
# a target name
config.add_transport(
    snmpEngine, udp.DOMAIN_NAME, udp.UdpAsyncioTransport().open_client_mode()
)
config.add_target_address(
    snmpEngine,
    "my-nms",
    udp.DOMAIN_NAME,
    ("127.0.0.1", 162),
    "my-creds",
    tagList="all-my-managers",
)

# Specify what kind of notification should be sent (TRAP or INFORM),
# to what targets (chosen by tag) and what filter should apply to
# the set of targets (selected by tag)
config.add_notification_target(
    snmpEngine, "my-notification", "my-filter", "all-my-managers", "trap"
)

# Allow NOTIFY access to Agent's MIB by this SNMP model (2), securityLevel
# and SecurityName
config.add_context(snmpEngine, "")
config.add_vacm_user(snmpEngine, 2, "my-area", "noAuthNoPriv", (), (), (1, 3, 6))

# *** SNMP engine configuration is complete by this line ***

# Create Notification Originator App instance.
ntfOrg = ntforg.NotificationOriginator()


# Error/confirmation receiver
# noinspection PyUnusedLocal,PyUnusedLocal,PyUnusedLocal,PyUnusedLocal,PyUnusedLocal
def cbFun(
    snmpEngine,
    sendRequestHandle,
    errorIndication,
    errorStatus,
    errorIndex,
    varBinds,
    cbCtx,
):
    print(
        "Notification {}, status - {}".format(
            sendRequestHandle, errorIndication and errorIndication or "delivered"
        )
    )


# Build and submit notification message to dispatcher
sendRequestHandle = ntfOrg.send_varbinds(
    snmpEngine,
    # Notification targets
    "my-notification",  # notification targets
    None,
    "",  # contextEngineId, contextName
    # var-binds
    [
        # SNMPv2-SMI::snmpTrapOID.0 = SNMPv2-MIB::coldStart
        (
            (1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0),
            v2c.ObjectIdentifier((1, 3, 6, 1, 6, 3, 1, 1, 5, 1)),
        ),
        # additional var-binds: ( (oid, value), ... )
        ((1, 3, 6, 1, 2, 1, 1, 1, 0), v2c.OctetString("Example Notificator")),
        ((1, 3, 6, 1, 2, 1, 1, 5, 0), v2c.OctetString("Notificator Example")),
    ],
    cbFun,
)

print("Notification %s is scheduled to be sent" % sendRequestHandle)

# Run I/O dispatcher which would send pending message and process response
snmpEngine.open_dispatcher()



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
import sys
from typing import TYPE_CHECKING


from pyasn1.codec.ber import encoder
from pyasn1.error import PyAsn1Error
from pysnmp import debug
from pysnmp.carrier.asyncio.dgram import udp, udp6
from pysnmp.proto import errind, error
from pysnmp.proto.secmod import base
from pysnmp.smi.error import NoSuchInstanceError

if TYPE_CHECKING:
    from pysnmp.entity.engine import SnmpEngine


class SnmpV1SecurityModel(base.AbstractSecurityModel):
    """Create SNMPv1 security model."""

    SECURITY_MODEL_ID = 1

    # According to rfc2576, community name <-> contextEngineId/contextName
    # mapping is up to MP module for notifications but belongs to secmod
    # responsibility for other PDU types. Since I do not yet understand
    # the reason for this de-coupling, I've moved this code from MP-scope
    # in here.

    def __init__(self):
        """Create SNMPv1 security model instance."""
        self.__transportBranchId = (
            self.__paramsBranchId
        ) = self.__communityBranchId = self.__securityBranchId = -1
        base.AbstractSecurityModel.__init__(self)

    def _close(self):
        """
        Close the security model to test memory leak.

        This method is intended for unit testing purposes only.
        It closes the security model and checks if all associated resources are released.
        """
        pass

    def _sec2com(
        self, snmpEngine: "SnmpEngine", securityName, contextEngineId, contextName
    ):
        (snmpTargetParamsSecurityName,) = snmpEngine.get_mib_builder().import_symbols(
            "SNMP-TARGET-MIB", "snmpTargetParamsSecurityName"
        )
        if self.__paramsBranchId != snmpTargetParamsSecurityName.branchVersionId:
            (
                snmpTargetParamsSecurityModel,
            ) = snmpEngine.get_mib_builder().import_symbols(
                "SNMP-TARGET-MIB", "snmpTargetParamsSecurityModel"
            )

            self.__nameToModelMap = {}

            nextMibNode = snmpTargetParamsSecurityName

            while True:
                try:
                    nextMibNode = snmpTargetParamsSecurityName.getNextNode(
                        nextMibNode.name
                    )

                except NoSuchInstanceError:
                    break

                instId = nextMibNode.name[len(snmpTargetParamsSecurityName.name) :]

                mibNode = snmpTargetParamsSecurityModel.getNode(
                    snmpTargetParamsSecurityModel.name + instId
                )

                try:
                    if mibNode.syntax not in self.__nameToModelMap:
                        self.__nameToModelMap[nextMibNode.syntax] = set()

                    self.__nameToModelMap[nextMibNode.syntax].add(mibNode.syntax)

                except PyAsn1Error:
                    debug.logger & debug.FLAG_SM and debug.logger(
                        "_sec2com: table entries {!r}/{!r} hashing failed".format(
                            nextMibNode.syntax, mibNode.syntax
                        )
                    )
                    continue

            self.__paramsBranchId = snmpTargetParamsSecurityName.branchVersionId

            # invalidate next map as it include this one
            self.__securityBranchId = -1

        (snmpCommunityName,) = snmpEngine.get_mib_builder().import_symbols(
            "SNMP-COMMUNITY-MIB", "snmpCommunityName"
        )
        if self.__securityBranchId != snmpCommunityName.branchVersionId:
            (
                snmpCommunitySecurityName,
                snmpCommunityContextEngineId,
                snmpCommunityContextName,
            ) = snmpEngine.get_mib_builder().import_symbols(
                "SNMP-COMMUNITY-MIB",
                "snmpCommunitySecurityName",
                "snmpCommunityContextEngineID",
                "snmpCommunityContextName",
            )

            self.__securityMap = {}

            nextMibNode = snmpCommunityName

            while True:
                try:
                    nextMibNode = snmpCommunityName.getNextNode(nextMibNode.name)

                except NoSuchInstanceError:
                    break

                instId = nextMibNode.name[len(snmpCommunityName.name) :]

                _securityName = snmpCommunitySecurityName.getNode(
                    snmpCommunitySecurityName.name + instId
                ).syntax

                _contextEngineId = snmpCommunityContextEngineId.getNode(
                    snmpCommunityContextEngineId.name + instId
                ).syntax

                _contextName = snmpCommunityContextName.getNode(
                    snmpCommunityContextName.name + instId
                ).syntax

                try:
                    self.__securityMap[
                        (_securityName, _contextEngineId, _contextName)
                    ] = nextMibNode.syntax

                except PyAsn1Error:
                    debug.logger & debug.FLAG_SM and debug.logger(
                        "_sec2com: table entries {!r}/{!r}/{!r} hashing failed".format(
                            _securityName, _contextEngineId, _contextName
                        )
                    )
                    continue

            self.__securityBranchId = snmpCommunityName.branchVersionId

            debug.logger & debug.FLAG_SM and debug.logger(
                "_sec2com: built securityName to communityName map, version {}: {}".format(
                    self.__securityBranchId, self.__securityMap
                )
            )

        try:
            return self.__securityMap[(securityName, contextEngineId, contextName)]

        except KeyError:
            raise error.StatusInformation(errorIndication=errind.unknownCommunityName)

    def _com2sec(self, snmpEngine: "SnmpEngine", communityName, transportInformation):
        (snmpTargetAddrTAddress,) = snmpEngine.get_mib_builder().import_symbols(
            "SNMP-TARGET-MIB", "snmpTargetAddrTAddress"
        )
        if self.__transportBranchId != snmpTargetAddrTAddress.branchVersionId:
            (
                SnmpTagValue,
                snmpTargetAddrTDomain,
                snmpTargetAddrTagList,
            ) = snmpEngine.get_mib_builder().import_symbols(
                "SNMP-TARGET-MIB",
                "SnmpTagValue",
                "snmpTargetAddrTDomain",
                "snmpTargetAddrTagList",
            )

            self.__emptyTag = SnmpTagValue("")

            self.__transportToTagMap = {}

            nextMibNode = snmpTargetAddrTagList

            while True:
                try:
                    nextMibNode = snmpTargetAddrTagList.getNextNode(nextMibNode.name)

                except NoSuchInstanceError:
                    break

                instId = nextMibNode.name[len(snmpTargetAddrTagList.name) :]
                targetAddrTDomain = snmpTargetAddrTDomain.getNode(
                    snmpTargetAddrTDomain.name + instId
                ).syntax
                targetAddrTAddress = snmpTargetAddrTAddress.getNode(
                    snmpTargetAddrTAddress.name + instId
                ).syntax

                targetAddrTDomain = tuple(targetAddrTDomain)

                if targetAddrTDomain[: len(udp.SNMP_UDP_DOMAIN)] == udp.SNMP_UDP_DOMAIN:
                    (SnmpUDPAddress,) = snmpEngine.get_mib_builder().import_symbols(
                        "SNMPv2-TM", "SnmpUDPAddress"
                    )
                    targetAddrTAddress = tuple(SnmpUDPAddress(targetAddrTAddress))
                elif (
                    targetAddrTDomain[: len(udp6.SNMP_UDP6_DOMAIN)]
                    == udp6.SNMP_UDP6_DOMAIN
                ):
                    (
                        TransportAddressIPv6,
                    ) = snmpEngine.get_mib_builder().import_symbols(
                        "TRANSPORT-ADDRESS-MIB", "TransportAddressIPv6"
                    )
                    targetAddrTAddress = tuple(TransportAddressIPv6(targetAddrTAddress))
                targetAddr = targetAddrTDomain, targetAddrTAddress
                targetAddrTagList = snmpTargetAddrTagList.getNode(
                    snmpTargetAddrTagList.name + instId
                ).syntax

                if targetAddr not in self.__transportToTagMap:
                    self.__transportToTagMap[targetAddr] = set()

                try:
                    if targetAddrTagList:
                        self.__transportToTagMap[targetAddr].update(
                            [
                                SnmpTagValue(x)
                                for x in targetAddrTagList.asOctets().split()
                            ]
                        )

                    else:
                        self.__transportToTagMap[targetAddr].add(self.__emptyTag)

                except PyAsn1Error:
                    debug.logger & debug.FLAG_SM and debug.logger(
                        "_com2sec: table entries {!r}/{!r} hashing failed".format(
                            targetAddr, targetAddrTagList
                        )
                    )
                    continue

            self.__transportBranchId = snmpTargetAddrTAddress.branchVersionId

            debug.logger & debug.FLAG_SM and debug.logger(
                "_com2sec: built transport-to-tag map version {}: {}".format(
                    self.__transportBranchId, self.__transportToTagMap
                )
            )

        (snmpTargetParamsSecurityName,) = snmpEngine.get_mib_builder().import_symbols(
            "SNMP-TARGET-MIB", "snmpTargetParamsSecurityName"
        )

        if self.__paramsBranchId != snmpTargetParamsSecurityName.branchVersionId:
            (
                snmpTargetParamsSecurityModel,
            ) = snmpEngine.get_mib_builder().import_symbols(
                "SNMP-TARGET-MIB", "snmpTargetParamsSecurityModel"
            )

            self.__nameToModelMap = {}

            nextMibNode = snmpTargetParamsSecurityName

            while True:
                try:
                    nextMibNode = snmpTargetParamsSecurityName.getNextNode(
                        nextMibNode.name
                    )

                except NoSuchInstanceError:
                    break

                instId = nextMibNode.name[len(snmpTargetParamsSecurityName.name) :]

                mibNode = snmpTargetParamsSecurityModel.getNode(
                    snmpTargetParamsSecurityModel.name + instId
                )

                try:
                    if nextMibNode.syntax not in self.__nameToModelMap:
                        self.__nameToModelMap[nextMibNode.syntax] = set()

                    self.__nameToModelMap[nextMibNode.syntax].add(mibNode.syntax)

                except PyAsn1Error:
                    debug.logger & debug.FLAG_SM and debug.logger(
                        "_com2sec: table entries {!r}/{!r} hashing failed".format(
                            nextMibNode.syntax, mibNode.syntax
                        )
                    )
                    continue

            self.__paramsBranchId = snmpTargetParamsSecurityName.branchVersionId

            # invalidate next map as it include this one
            self.__communityBranchId = -1

            debug.logger & debug.FLAG_SM and debug.logger(
                "_com2sec: built securityName to securityModel map, version {}: {}".format(
                    self.__paramsBranchId, self.__nameToModelMap
                )
            )

        (snmpCommunityName,) = snmpEngine.get_mib_builder().import_symbols(
            "SNMP-COMMUNITY-MIB", "snmpCommunityName"
        )
        if self.__communityBranchId != snmpCommunityName.branchVersionId:
            (
                snmpCommunitySecurityName,
                snmpCommunityContextEngineId,
                snmpCommunityContextName,
                snmpCommunityTransportTag,
            ) = snmpEngine.get_mib_builder().import_symbols(
                "SNMP-COMMUNITY-MIB",
                "snmpCommunitySecurityName",
                "snmpCommunityContextEngineID",
                "snmpCommunityContextName",
                "snmpCommunityTransportTag",
            )

            self.__communityToTagMap = {}
            self.__tagAndCommunityToSecurityMap = {}

            nextMibNode = snmpCommunityName

            while True:
                try:
                    nextMibNode = snmpCommunityName.getNextNode(nextMibNode.name)

                except NoSuchInstanceError:
                    break

                instId = nextMibNode.name[len(snmpCommunityName.name) :]

                securityName = snmpCommunitySecurityName.getNode(
                    snmpCommunitySecurityName.name + instId
                ).syntax

                contextEngineId = snmpCommunityContextEngineId.getNode(
                    snmpCommunityContextEngineId.name + instId
                ).syntax

                contextName = snmpCommunityContextName.getNode(
                    snmpCommunityContextName.name + instId
                ).syntax

                transportTag = snmpCommunityTransportTag.getNode(
                    snmpCommunityTransportTag.name + instId
                ).syntax

                _tagAndCommunity = transportTag, nextMibNode.syntax

                try:
                    if _tagAndCommunity not in self.__tagAndCommunityToSecurityMap:
                        self.__tagAndCommunityToSecurityMap[_tagAndCommunity] = set()

                    self.__tagAndCommunityToSecurityMap[_tagAndCommunity].add(
                        (securityName, contextEngineId, contextName)
                    )

                    if nextMibNode.syntax not in self.__communityToTagMap:
                        self.__communityToTagMap[nextMibNode.syntax] = set()

                    self.__communityToTagMap[nextMibNode.syntax].add(transportTag)

                except PyAsn1Error:
                    debug.logger & debug.FLAG_SM and debug.logger(
                        "_com2sec: table entries {!r}/{!r} hashing failed".format(
                            _tagAndCommunity, nextMibNode.syntax
                        )
                    )
                    continue

            self.__communityBranchId = snmpCommunityName.branchVersionId

            debug.logger & debug.FLAG_SM and debug.logger(
                "_com2sec: built communityName to tag map (securityModel {}), version {}: {}".format(
                    self.SECURITY_MODEL_ID,
                    self.__communityBranchId,
                    self.__communityToTagMap,
                )
            )

            debug.logger & debug.FLAG_SM and debug.logger(
                "_com2sec: built tag & community to securityName map (securityModel {}), version {}: {}".format(
                    self.SECURITY_MODEL_ID,
                    self.__communityBranchId,
                    self.__tagAndCommunityToSecurityMap,
                )
            )

        if communityName in self.__communityToTagMap:
            if transportInformation in self.__transportToTagMap:
                tags = self.__transportToTagMap[transportInformation].intersection(
                    self.__communityToTagMap[communityName]
                )
            elif self.__emptyTag in self.__communityToTagMap[communityName]:
                tags = [self.__emptyTag]
            else:
                raise error.StatusInformation(
                    errorIndication=errind.unknownCommunityName
                )

            candidateSecurityNames = []

            for x in [
                self.__tagAndCommunityToSecurityMap[(t, communityName)] for t in tags
            ]:
                candidateSecurityNames.extend(list(x))

            # 5.2.1 (row selection in snmpCommunityTable)
            # Picks first match but favors entries already in targets table
            if candidateSecurityNames:
                candidateSecurityNames.sort(
                    key=lambda x, m=self.__nameToModelMap, v=self.SECURITY_MODEL_ID: (
                        not int(x[0] in m and v in m[x[0]]),
                        str(x[0]),
                    )
                )
                chosenSecurityName = candidateSecurityNames[0]  # min()
                debug.logger & debug.FLAG_SM and debug.logger(
                    "_com2sec: securityName candidates for communityName '{}' are {}; choosing securityName '{}'".format(
                        communityName, candidateSecurityNames, chosenSecurityName[0]
                    )
                )
                return chosenSecurityName

        raise error.StatusInformation(errorIndication=errind.unknownCommunityName)

    def generate_request_message(
        self,
        snmpEngine: "SnmpEngine",
        messageProcessingModel,
        globalData,
        maxMessageSize,
        securityModel,
        securityEngineId,
        securityName,
        securityLevel,
        scopedPDU,
    ):
        """Generate a request message."""
        (msg,) = globalData
        contextEngineId, contextName, pdu = scopedPDU

        # rfc2576: 5.2.3
        communityName = self._sec2com(
            snmpEngine, securityName, contextEngineId, contextName
        )

        debug.logger & debug.FLAG_SM and debug.logger(
            "generateRequestMsg: using community {!r} for securityModel {!r}, securityName {!r}, contextEngineId {!r} contextName {!r}".format(
                communityName, securityModel, securityName, contextEngineId, contextName
            )
        )

        securityParameters = communityName

        msg.setComponentByPosition(1, securityParameters)
        msg.setComponentByPosition(2)
        msg.getComponentByPosition(2).setComponentByType(
            pdu.tagSet,
            pdu,
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )

        debug.logger & debug.FLAG_MP and debug.logger(
            f"generateRequestMsg: {msg.prettyPrint()}"
        )

        try:
            return securityParameters, encoder.encode(msg)

        except PyAsn1Error:
            debug.logger & debug.FLAG_MP and debug.logger(
                "generateRequestMsg: serialization failure: %s" % sys.exc_info()[1]
            )
            raise error.StatusInformation(errorIndication=errind.serializationError)

    def generate_response_message(
        self,
        snmpEngine: "SnmpEngine",
        messageProcessingModel,
        globalData,
        maxMessageSize,
        securityModel,
        securityEngineID,
        securityName,
        securityLevel,
        scopedPDU,
        securityStateReference,
        ctx,
    ):
        """Generate a response message."""
        # rfc2576: 5.2.2
        (msg,) = globalData
        contextEngineId, contextName, pdu = scopedPDU
        cachedSecurityData = self._cache.pop(securityStateReference)
        communityName = cachedSecurityData["communityName"]

        debug.logger & debug.FLAG_SM and debug.logger(
            "generateResponseMsg: recovered community {!r} by securityStateReference {}".format(
                communityName, securityStateReference
            )
        )

        msg.setComponentByPosition(1, communityName)
        msg.setComponentByPosition(2)
        msg.getComponentByPosition(2).setComponentByType(
            pdu.tagSet,
            pdu,
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )

        debug.logger & debug.FLAG_MP and debug.logger(
            f"generateResponseMsg: {msg.prettyPrint()}"
        )

        try:
            return communityName, encoder.encode(msg)

        except PyAsn1Error:
            debug.logger & debug.FLAG_MP and debug.logger(
                "generateResponseMsg: serialization failure: %s" % sys.exc_info()[1]
            )
            raise error.StatusInformation(errorIndication=errind.serializationError)

    def process_incoming_message(
        self,
        snmpEngine: "SnmpEngine",
        messageProcessingModel,
        maxMessageSize,
        securityParameters,
        securityModel,
        securityLevel,
        wholeMsg,
        msg,
    ):
        """Process an incoming message."""
        # rfc2576: 5.2.1
        communityName, transportInformation = securityParameters

        scope = dict(
            communityName=communityName, transportInformation=transportInformation
        )

        snmpEngine.observer.store_execution_context(
            snmpEngine, "rfc2576.processIncomingMsg:writable", scope
        )
        snmpEngine.observer.clear_execution_context(
            snmpEngine, "rfc2576.processIncomingMsg:writable"
        )

        try:
            securityName, contextEngineId, contextName = self._com2sec(
                snmpEngine,
                scope.get("communityName", communityName),
                scope.get("transportInformation", transportInformation),
            )

        except error.StatusInformation:
            (snmpInBadCommunityNames,) = snmpEngine.get_mib_builder().import_symbols(
                "__SNMPv2-MIB", "snmpInBadCommunityNames"
            )
            snmpInBadCommunityNames.syntax += 1
            raise error.StatusInformation(
                errorIndication=errind.unknownCommunityName, communityName=communityName
            )

        (snmpEngineID,) = snmpEngine.get_mib_builder().import_symbols(
            "__SNMP-FRAMEWORK-MIB", "snmpEngineID"
        )

        securityEngineID = snmpEngineID.syntax

        snmpEngine.observer.store_execution_context(
            snmpEngine,
            "rfc2576.processIncomingMsg",
            dict(
                transportInformation=transportInformation,
                securityEngineId=securityEngineID,
                securityName=securityName,
                communityName=communityName,
                contextEngineId=contextEngineId,
                contextName=contextName,
            ),
        )
        snmpEngine.observer.clear_execution_context(
            snmpEngine, "rfc2576.processIncomingMsg"
        )

        debug.logger & debug.FLAG_SM and debug.logger(
            "processIncomingMsg: looked up securityName {!r} securityModel {!r} contextEngineId {!r} contextName {!r} by communityName {!r} AND transportInformation {!r}".format(
                securityName,
                self.SECURITY_MODEL_ID,
                contextEngineId,
                contextName,
                communityName,
                transportInformation,
            )
        )

        stateReference = self._cache.push(communityName=communityName)

        scopedPDU = (
            contextEngineId,
            contextName,
            msg.getComponentByPosition(2).getComponent(),
        )
        maxSizeResponseScopedPDU = maxMessageSize - 128
        securityStateReference = stateReference

        debug.logger & debug.FLAG_SM and debug.logger(
            "processIncomingMsg: generated maxSizeResponseScopedPDU {} securityStateReference {}".format(
                maxSizeResponseScopedPDU, securityStateReference
            )
        )

        return (
            securityEngineID,
            securityName,
            scopedPDU,
            maxSizeResponseScopedPDU,
            securityStateReference,
        )


class SnmpV2cSecurityModel(SnmpV1SecurityModel):
    """Create SNMPv2c security model."""

    SECURITY_MODEL_ID = 2


# XXX
# contextEngineId/contextName goes to globalData



"""
Report SNMP engine processing details
+++++++++++++++++++++++++++++++++++++

Send SNMP GET request with the following options:

* with SNMPv3 with user 'usr-sha-aes', SHA auth and AES128 privacy protocols
* over IPv4/UDP
* to an Agent at 127.0.0.1:161
* for an OID in tuple form
* also registers its own execution observer to snmpEngine

While execution, this script will report some details on request processing
as seen by rfc3412.sendPdu() and rfc3412.receiveMessage() abstract interfaces.

This script performs similar to the following Net-SNMP command:

| $ snmpget -v3 -l authPriv -u usr-sha-aes -a SHA -A authkey1 -x AES -X privkey1 -ObentU 127.0.0.1:161  1.3.6.1.2.1.1.1.0

"""  #

from pysnmp.entity import engine, config
from pysnmp.carrier.asyncio.dgram import udp
from pysnmp.entity.rfc3413 import cmdgen

# Create SNMP engine instance
snmpEngine = engine.SnmpEngine()


# Execution point observer setup


# Register a callback to be invoked at specified execution point of
# SNMP Engine and passed local variables at code point's local scope
# noinspection PyUnusedLocal,PyUnusedLocal
def requestObserver(snmpEngine, execpoint, variables, cbCtx):
    print("Execution point: %s" % execpoint)
    print(
        "* transportDomain: %s"
        % ".".join([str(x) for x in variables["transportDomain"]])
    )
    print(
        "* transportAddress: %s"
        % "@".join([str(x) for x in variables["transportAddress"]])
    )
    print("* securityModel: %s" % variables["securityModel"])
    print("* securityName: %s" % variables["securityName"])
    print("* securityLevel: %s" % variables["securityLevel"])
    print(
        "* contextEngineId: {}".format(
            variables["contextEngineId"]
            and variables["contextEngineId"].prettyPrint()
            or "<empty>"
        )
    )
    print("* contextName: %s" % variables["contextName"].prettyPrint())
    print("* PDU: %s" % variables["pdu"].prettyPrint())


snmpEngine.observer.register_observer(
    requestObserver, "rfc3412.sendPdu", "rfc3412.receiveMessage:response"
)

#
# SNMPv3/USM setup
#

# user: usr-sha-aes, auth: SHA, priv AES
config.add_v3_user(
    snmpEngine,
    "usr-sha-aes",
    config.USM_AUTH_HMAC96_SHA,
    "authkey1",
    config.USM_PRIV_CFB128_AES,
    "privkey1",
)
config.add_target_parameters(snmpEngine, "my-creds", "usr-sha-aes", "authPriv")

#
# Setup transport endpoint and bind it with security settings yielding
# a target name
#

# UDP/IPv4
config.add_transport(
    snmpEngine, udp.DOMAIN_NAME, udp.UdpAsyncioTransport().open_client_mode()
)
config.add_target_address(
    snmpEngine, "my-router", udp.DOMAIN_NAME, ("127.0.0.1", 161), "my-creds"
)


# Error/response receiver
# noinspection PyUnusedLocal,PyUnusedLocal,PyUnusedLocal
def cbFun(
    snmpEngine,
    sendRequestHandle,
    errorIndication,
    errorStatus,
    errorIndex,
    varBinds,
    cbCtx,
):
    if errorIndication:
        print(errorIndication)
    elif errorStatus:
        print(
            f"{errorStatus.prettyPrint()} at {varBinds[int(errorIndex) - 1][0] if errorIndex else '?'}"
        )
    else:
        for oid, val in varBinds:
            print(f"{oid.prettyPrint()} = {val.prettyPrint()}")


# Prepare and send a request message
cmdgen.GetCommandGenerator().send_varbinds(
    snmpEngine,
    "my-router",
    None,
    "",  # contextEngineId, contextName
    [((1, 3, 6, 1, 2, 1, 1, 1, 0), None)],
    cbFun,
)

# Run I/O dispatcher which would send pending queries and process responses
snmpEngine.open_dispatcher(3)

snmpEngine.observer.unregisterObserver()

snmpEngine.close_dispatcher()



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
# PySNMP MIB module SNMPv2-MIB (https://www.pysnmp.com/pysnmp)
# ASN.1 source file://asn1/SNMPv2-MIB
# Produced by pysmi-1.5.8 at Sat Nov  2 15:25:43 2024
# On host MacBook-Pro.local platform Darwin version 24.1.0 by user lextm
# Using Python version 3.12.0 (main, Nov 14 2023, 23:52:11) [Clang 15.0.0 (clang-1500.0.40.1)]
#

# Import base ASN.1 objects even if this MIB does not use it

(Integer,
 OctetString,
 ObjectIdentifier) = mibBuilder.import_symbols(
    "ASN1",
    "Integer",
    "OctetString",
    "ObjectIdentifier")

(NamedValues,) = mibBuilder.import_symbols(
    "ASN1-ENUMERATION",
    "NamedValues")
(ConstraintsIntersection,
 ConstraintsUnion,
 SingleValueConstraint,
 ValueRangeConstraint,
 ValueSizeConstraint) = mibBuilder.import_symbols(
    "ASN1-REFINEMENT",
    "ConstraintsIntersection",
    "ConstraintsUnion",
    "SingleValueConstraint",
    "ValueRangeConstraint",
    "ValueSizeConstraint")

# Import SMI symbols from the MIBs this MIB depends on

(ModuleCompliance,
 NotificationGroup,
 ObjectGroup) = mibBuilder.import_symbols(
    "SNMPv2-CONF",
    "ModuleCompliance",
    "NotificationGroup",
    "ObjectGroup")

(Bits,
 Counter32,
 Counter64,
 Gauge32,
 Integer32,
 IpAddress,
 ModuleIdentity,
 MibIdentifier,
 NotificationType,
 ObjectIdentity,
 MibScalar,
 MibTable,
 MibTableRow,
 MibTableColumn,
 TimeTicks,
 Unsigned32,
 iso,
 mib_2,
 snmpModules) = mibBuilder.import_symbols(
    "SNMPv2-SMI",
    "Bits",
    "Counter32",
    "Counter64",
    "Gauge32",
    "Integer32",
    "IpAddress",
    "ModuleIdentity",
    "MibIdentifier",
    "NotificationType",
    "ObjectIdentity",
    "MibScalar",
    "MibTable",
    "MibTableRow",
    "MibTableColumn",
    "TimeTicks",
    "Unsigned32",
    "iso",
    "mib-2",
    "snmpModules")

(DisplayString,
 TextualConvention,
 TestAndIncr,
 TimeStamp) = mibBuilder.import_symbols(
    "SNMPv2-TC",
    "DisplayString",
    "TextualConvention",
    "TestAndIncr",
    "TimeStamp")


# MODULE-IDENTITY

snmpMIB = ModuleIdentity(
    (1, 3, 6, 1, 6, 3, 1)
)
if mibBuilder.loadTexts:
    snmpMIB.setRevisions(
        ("2002-10-16 00:00",
         "1995-11-09 00:00",
         "1993-04-01 00:00")
    )
if mibBuilder.loadTexts:
    snmpMIB.setLastUpdated("200210160000Z")
if mibBuilder.loadTexts:
    snmpMIB.setOrganization("IETF SNMPv3 Working Group")
if mibBuilder.loadTexts:
    snmpMIB.setContactInfo("WG-EMail: snmpv3@lists.tislabs.com Subscribe: snmpv3-request@lists.tislabs.com Co-Chair: Russ Mundy Network Associates Laboratories postal: 15204 Omega Drive, Suite 300 Rockville, MD 20850-4601 USA EMail: mundy@tislabs.com phone: +1 301 947-7107 Co-Chair: David Harrington Enterasys Networks postal: 35 Industrial Way P. O. Box 5005 Rochester, NH 03866-5005 USA EMail: dbh@enterasys.com phone: +1 603 337-2614 Editor: Randy Presuhn BMC Software, Inc. postal: 2141 North First Street San Jose, CA 95131 USA EMail: randy_presuhn@bmc.com phone: +1 408 546-1006")
if mibBuilder.loadTexts:
    snmpMIB.setDescription("The MIB module for SNMP entities. Copyright (C) The Internet Society (2002). This version of this MIB module is part of RFC 3418; see the RFC itself for full legal notices. ")


# Types definitions


# TEXTUAL-CONVENTIONS



# MIB Managed Objects in the order of their OIDs

_System_ObjectIdentity = ObjectIdentity
system = _System_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 1)
)


class _SysDescr_Type(DisplayString):
    """Custom type sysDescr based on DisplayString"""
    subtypeSpec = DisplayString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 255),
    )


_SysDescr_Type.__name__ = "DisplayString"
_SysDescr_Object = MibScalar
sysDescr = _SysDescr_Object(
    (1, 3, 6, 1, 2, 1, 1, 1),
    _SysDescr_Type()
)
sysDescr.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    sysDescr.setStatus("current")
if mibBuilder.loadTexts:
    sysDescr.setDescription("A textual description of the entity. This value should include the full name and version identification of the system's hardware type, software operating-system, and networking software.")
_SysObjectID_Type = ObjectIdentifier
_SysObjectID_Object = MibScalar
sysObjectID = _SysObjectID_Object(
    (1, 3, 6, 1, 2, 1, 1, 2),
    _SysObjectID_Type()
)
sysObjectID.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    sysObjectID.setStatus("current")
if mibBuilder.loadTexts:
    sysObjectID.setDescription("The vendor's authoritative identification of the network management subsystem contained in the entity. This value is allocated within the SMI enterprises subtree (1.3.6.1.4.1) and provides an easy and unambiguous means for determining `what kind of box' is being managed. For example, if vendor `Flintstones, Inc.' was assigned the subtree 1.3.6.1.4.1.424242, it could assign the identifier 1.3.6.1.4.1.424242.1.1 to its `Fred Router'.")
_SysUpTime_Type = TimeTicks
_SysUpTime_Object = MibScalar
sysUpTime = _SysUpTime_Object(
    (1, 3, 6, 1, 2, 1, 1, 3),
    _SysUpTime_Type()
)
sysUpTime.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    sysUpTime.setStatus("current")
if mibBuilder.loadTexts:
    sysUpTime.setDescription("The time (in hundredths of a second) since the network management portion of the system was last re-initialized.")


class _SysContact_Type(DisplayString):
    """Custom type sysContact based on DisplayString"""
    subtypeSpec = DisplayString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 255),
    )


_SysContact_Type.__name__ = "DisplayString"
_SysContact_Object = MibScalar
sysContact = _SysContact_Object(
    (1, 3, 6, 1, 2, 1, 1, 4),
    _SysContact_Type()
)
sysContact.setMaxAccess("read-write")
if mibBuilder.loadTexts:
    sysContact.setStatus("current")
if mibBuilder.loadTexts:
    sysContact.setDescription("The textual identification of the contact person for this managed node, together with information on how to contact this person. If no contact information is known, the value is the zero-length string.")


class _SysName_Type(DisplayString):
    """Custom type sysName based on DisplayString"""
    subtypeSpec = DisplayString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 255),
    )


_SysName_Type.__name__ = "DisplayString"
_SysName_Object = MibScalar
sysName = _SysName_Object(
    (1, 3, 6, 1, 2, 1, 1, 5),
    _SysName_Type()
)
sysName.setMaxAccess("read-write")
if mibBuilder.loadTexts:
    sysName.setStatus("current")
if mibBuilder.loadTexts:
    sysName.setDescription("An administratively-assigned name for this managed node. By convention, this is the node's fully-qualified domain name. If the name is unknown, the value is the zero-length string.")


class _SysLocation_Type(DisplayString):
    """Custom type sysLocation based on DisplayString"""
    subtypeSpec = DisplayString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 255),
    )


_SysLocation_Type.__name__ = "DisplayString"
_SysLocation_Object = MibScalar
sysLocation = _SysLocation_Object(
    (1, 3, 6, 1, 2, 1, 1, 6),
    _SysLocation_Type()
)
sysLocation.setMaxAccess("read-write")
if mibBuilder.loadTexts:
    sysLocation.setStatus("current")
if mibBuilder.loadTexts:
    sysLocation.setDescription("The physical location of this node (e.g., 'telephone closet, 3rd floor'). If the location is unknown, the value is the zero-length string.")


class _SysServices_Type(Integer32):
    """Custom type sysServices based on Integer32"""
    subtypeSpec = Integer32.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueRangeConstraint(0, 127),
    )


_SysServices_Type.__name__ = "Integer32"
_SysServices_Object = MibScalar
sysServices = _SysServices_Object(
    (1, 3, 6, 1, 2, 1, 1, 7),
    _SysServices_Type()
)
sysServices.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    sysServices.setStatus("current")
if mibBuilder.loadTexts:
    sysServices.setDescription("A value which indicates the set of services that this entity may potentially offer. The value is a sum. This sum initially takes the value zero. Then, for each layer, L, in the range 1 through 7, that this node performs transactions for, 2 raised to (L - 1) is added to the sum. For example, a node which performs only routing functions would have a value of 4 (2^(3-1)). In contrast, a node which is a host offering application services would have a value of 72 (2^(4-1) + 2^(7-1)). Note that in the context of the Internet suite of protocols, values should be calculated accordingly: layer functionality 1 physical (e.g., repeaters) 2 datalink/subnetwork (e.g., bridges) 3 internet (e.g., supports the IP) 4 end-to-end (e.g., supports the TCP) 7 applications (e.g., supports the SMTP) For systems including OSI protocols, layers 5 and 6 may also be counted.")
_SysORLastChange_Type = TimeStamp
_SysORLastChange_Object = MibScalar
sysORLastChange = _SysORLastChange_Object(
    (1, 3, 6, 1, 2, 1, 1, 8),
    _SysORLastChange_Type()
)
sysORLastChange.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    sysORLastChange.setStatus("current")
if mibBuilder.loadTexts:
    sysORLastChange.setDescription("The value of sysUpTime at the time of the most recent change in state or value of any instance of sysORID.")
_SysORTable_Object = MibTable
sysORTable = _SysORTable_Object(
    (1, 3, 6, 1, 2, 1, 1, 9)
)
if mibBuilder.loadTexts:
    sysORTable.setStatus("current")
if mibBuilder.loadTexts:
    sysORTable.setDescription("The (conceptual) table listing the capabilities of the local SNMP application acting as a command responder with respect to various MIB modules. SNMP entities having dynamically-configurable support of MIB modules will have a dynamically-varying number of conceptual rows.")
_SysOREntry_Object = MibTableRow
sysOREntry = _SysOREntry_Object(
    (1, 3, 6, 1, 2, 1, 1, 9, 1)
)
sysOREntry.setIndexNames(
    (0, "SNMPv2-MIB", "sysORIndex"),
)
if mibBuilder.loadTexts:
    sysOREntry.setStatus("current")
if mibBuilder.loadTexts:
    sysOREntry.setDescription("An entry (conceptual row) in the sysORTable.")


class _SysORIndex_Type(Integer32):
    """Custom type sysORIndex based on Integer32"""
    subtypeSpec = Integer32.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueRangeConstraint(1, 2147483647),
    )


_SysORIndex_Type.__name__ = "Integer32"
_SysORIndex_Object = MibTableColumn
sysORIndex = _SysORIndex_Object(
    (1, 3, 6, 1, 2, 1, 1, 9, 1, 1),
    _SysORIndex_Type()
)
sysORIndex.setMaxAccess("not-accessible")
if mibBuilder.loadTexts:
    sysORIndex.setStatus("current")
if mibBuilder.loadTexts:
    sysORIndex.setDescription("The auxiliary variable used for identifying instances of the columnar objects in the sysORTable.")
_SysORID_Type = ObjectIdentifier
_SysORID_Object = MibTableColumn
sysORID = _SysORID_Object(
    (1, 3, 6, 1, 2, 1, 1, 9, 1, 2),
    _SysORID_Type()
)
sysORID.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    sysORID.setStatus("current")
if mibBuilder.loadTexts:
    sysORID.setDescription("An authoritative identification of a capabilities statement with respect to various MIB modules supported by the local SNMP application acting as a command responder.")
_SysORDescr_Type = DisplayString
_SysORDescr_Object = MibTableColumn
sysORDescr = _SysORDescr_Object(
    (1, 3, 6, 1, 2, 1, 1, 9, 1, 3),
    _SysORDescr_Type()
)
sysORDescr.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    sysORDescr.setStatus("current")
if mibBuilder.loadTexts:
    sysORDescr.setDescription("A textual description of the capabilities identified by the corresponding instance of sysORID.")
_SysORUpTime_Type = TimeStamp
_SysORUpTime_Object = MibTableColumn
sysORUpTime = _SysORUpTime_Object(
    (1, 3, 6, 1, 2, 1, 1, 9, 1, 4),
    _SysORUpTime_Type()
)
sysORUpTime.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    sysORUpTime.setStatus("current")
if mibBuilder.loadTexts:
    sysORUpTime.setDescription("The value of sysUpTime at the time this conceptual row was last instantiated.")
_Snmp_ObjectIdentity = ObjectIdentity
snmp = _Snmp_ObjectIdentity(
    (1, 3, 6, 1, 2, 1, 11)
)
_SnmpInPkts_Type = Counter32
_SnmpInPkts_Object = MibScalar
snmpInPkts = _SnmpInPkts_Object(
    (1, 3, 6, 1, 2, 1, 11, 1),
    _SnmpInPkts_Type()
)
snmpInPkts.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInPkts.setStatus("current")
if mibBuilder.loadTexts:
    snmpInPkts.setDescription("The total number of messages delivered to the SNMP entity from the transport service.")
_SnmpOutPkts_Type = Counter32
_SnmpOutPkts_Object = MibScalar
snmpOutPkts = _SnmpOutPkts_Object(
    (1, 3, 6, 1, 2, 1, 11, 2),
    _SnmpOutPkts_Type()
)
snmpOutPkts.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutPkts.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutPkts.setDescription("The total number of SNMP Messages which were passed from the SNMP protocol entity to the transport service.")
_SnmpInBadVersions_Type = Counter32
_SnmpInBadVersions_Object = MibScalar
snmpInBadVersions = _SnmpInBadVersions_Object(
    (1, 3, 6, 1, 2, 1, 11, 3),
    _SnmpInBadVersions_Type()
)
snmpInBadVersions.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInBadVersions.setStatus("current")
if mibBuilder.loadTexts:
    snmpInBadVersions.setDescription("The total number of SNMP messages which were delivered to the SNMP entity and were for an unsupported SNMP version.")
_SnmpInBadCommunityNames_Type = Counter32
_SnmpInBadCommunityNames_Object = MibScalar
snmpInBadCommunityNames = _SnmpInBadCommunityNames_Object(
    (1, 3, 6, 1, 2, 1, 11, 4),
    _SnmpInBadCommunityNames_Type()
)
snmpInBadCommunityNames.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInBadCommunityNames.setStatus("current")
if mibBuilder.loadTexts:
    snmpInBadCommunityNames.setDescription("The total number of community-based SNMP messages (for example, SNMPv1) delivered to the SNMP entity which used an SNMP community name not known to said entity. Also, implementations which authenticate community-based SNMP messages using check(s) in addition to matching the community name (for example, by also checking whether the message originated from a transport address allowed to use a specified community name) MAY include in this value the number of messages which failed the additional check(s). It is strongly RECOMMENDED that the documentation for any security model which is used to authenticate community-based SNMP messages specify the precise conditions that contribute to this value.")
_SnmpInBadCommunityUses_Type = Counter32
_SnmpInBadCommunityUses_Object = MibScalar
snmpInBadCommunityUses = _SnmpInBadCommunityUses_Object(
    (1, 3, 6, 1, 2, 1, 11, 5),
    _SnmpInBadCommunityUses_Type()
)
snmpInBadCommunityUses.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInBadCommunityUses.setStatus("current")
if mibBuilder.loadTexts:
    snmpInBadCommunityUses.setDescription("The total number of community-based SNMP messages (for example, SNMPv1) delivered to the SNMP entity which represented an SNMP operation that was not allowed for the SNMP community named in the message. The precise conditions under which this counter is incremented (if at all) depend on how the SNMP entity implements its access control mechanism and how its applications interact with that access control mechanism. It is strongly RECOMMENDED that the documentation for any access control mechanism which is used to control access to and visibility of MIB instrumentation specify the precise conditions that contribute to this value.")
_SnmpInASNParseErrs_Type = Counter32
_SnmpInASNParseErrs_Object = MibScalar
snmpInASNParseErrs = _SnmpInASNParseErrs_Object(
    (1, 3, 6, 1, 2, 1, 11, 6),
    _SnmpInASNParseErrs_Type()
)
snmpInASNParseErrs.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInASNParseErrs.setStatus("current")
if mibBuilder.loadTexts:
    snmpInASNParseErrs.setDescription("The total number of ASN.1 or BER errors encountered by the SNMP entity when decoding received SNMP messages.")
_SnmpInTooBigs_Type = Counter32
_SnmpInTooBigs_Object = MibScalar
snmpInTooBigs = _SnmpInTooBigs_Object(
    (1, 3, 6, 1, 2, 1, 11, 8),
    _SnmpInTooBigs_Type()
)
snmpInTooBigs.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInTooBigs.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInTooBigs.setDescription("The total number of SNMP PDUs which were delivered to the SNMP protocol entity and for which the value of the error-status field was `tooBig'.")
_SnmpInNoSuchNames_Type = Counter32
_SnmpInNoSuchNames_Object = MibScalar
snmpInNoSuchNames = _SnmpInNoSuchNames_Object(
    (1, 3, 6, 1, 2, 1, 11, 9),
    _SnmpInNoSuchNames_Type()
)
snmpInNoSuchNames.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInNoSuchNames.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInNoSuchNames.setDescription("The total number of SNMP PDUs which were delivered to the SNMP protocol entity and for which the value of the error-status field was `noSuchName'.")
_SnmpInBadValues_Type = Counter32
_SnmpInBadValues_Object = MibScalar
snmpInBadValues = _SnmpInBadValues_Object(
    (1, 3, 6, 1, 2, 1, 11, 10),
    _SnmpInBadValues_Type()
)
snmpInBadValues.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInBadValues.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInBadValues.setDescription("The total number of SNMP PDUs which were delivered to the SNMP protocol entity and for which the value of the error-status field was `badValue'.")
_SnmpInReadOnlys_Type = Counter32
_SnmpInReadOnlys_Object = MibScalar
snmpInReadOnlys = _SnmpInReadOnlys_Object(
    (1, 3, 6, 1, 2, 1, 11, 11),
    _SnmpInReadOnlys_Type()
)
snmpInReadOnlys.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInReadOnlys.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInReadOnlys.setDescription("The total number valid SNMP PDUs which were delivered to the SNMP protocol entity and for which the value of the error-status field was `readOnly'. It should be noted that it is a protocol error to generate an SNMP PDU which contains the value `readOnly' in the error-status field, as such this object is provided as a means of detecting incorrect implementations of the SNMP.")
_SnmpInGenErrs_Type = Counter32
_SnmpInGenErrs_Object = MibScalar
snmpInGenErrs = _SnmpInGenErrs_Object(
    (1, 3, 6, 1, 2, 1, 11, 12),
    _SnmpInGenErrs_Type()
)
snmpInGenErrs.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInGenErrs.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInGenErrs.setDescription("The total number of SNMP PDUs which were delivered to the SNMP protocol entity and for which the value of the error-status field was `genErr'.")
_SnmpInTotalReqVars_Type = Counter32
_SnmpInTotalReqVars_Object = MibScalar
snmpInTotalReqVars = _SnmpInTotalReqVars_Object(
    (1, 3, 6, 1, 2, 1, 11, 13),
    _SnmpInTotalReqVars_Type()
)
snmpInTotalReqVars.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInTotalReqVars.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInTotalReqVars.setDescription("The total number of MIB objects which have been retrieved successfully by the SNMP protocol entity as the result of receiving valid SNMP Get-Request and Get-Next PDUs.")
_SnmpInTotalSetVars_Type = Counter32
_SnmpInTotalSetVars_Object = MibScalar
snmpInTotalSetVars = _SnmpInTotalSetVars_Object(
    (1, 3, 6, 1, 2, 1, 11, 14),
    _SnmpInTotalSetVars_Type()
)
snmpInTotalSetVars.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInTotalSetVars.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInTotalSetVars.setDescription("The total number of MIB objects which have been altered successfully by the SNMP protocol entity as the result of receiving valid SNMP Set-Request PDUs.")
_SnmpInGetRequests_Type = Counter32
_SnmpInGetRequests_Object = MibScalar
snmpInGetRequests = _SnmpInGetRequests_Object(
    (1, 3, 6, 1, 2, 1, 11, 15),
    _SnmpInGetRequests_Type()
)
snmpInGetRequests.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInGetRequests.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInGetRequests.setDescription("The total number of SNMP Get-Request PDUs which have been accepted and processed by the SNMP protocol entity.")
_SnmpInGetNexts_Type = Counter32
_SnmpInGetNexts_Object = MibScalar
snmpInGetNexts = _SnmpInGetNexts_Object(
    (1, 3, 6, 1, 2, 1, 11, 16),
    _SnmpInGetNexts_Type()
)
snmpInGetNexts.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInGetNexts.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInGetNexts.setDescription("The total number of SNMP Get-Next PDUs which have been accepted and processed by the SNMP protocol entity.")
_SnmpInSetRequests_Type = Counter32
_SnmpInSetRequests_Object = MibScalar
snmpInSetRequests = _SnmpInSetRequests_Object(
    (1, 3, 6, 1, 2, 1, 11, 17),
    _SnmpInSetRequests_Type()
)
snmpInSetRequests.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInSetRequests.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInSetRequests.setDescription("The total number of SNMP Set-Request PDUs which have been accepted and processed by the SNMP protocol entity.")
_SnmpInGetResponses_Type = Counter32
_SnmpInGetResponses_Object = MibScalar
snmpInGetResponses = _SnmpInGetResponses_Object(
    (1, 3, 6, 1, 2, 1, 11, 18),
    _SnmpInGetResponses_Type()
)
snmpInGetResponses.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInGetResponses.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInGetResponses.setDescription("The total number of SNMP Get-Response PDUs which have been accepted and processed by the SNMP protocol entity.")
_SnmpInTraps_Type = Counter32
_SnmpInTraps_Object = MibScalar
snmpInTraps = _SnmpInTraps_Object(
    (1, 3, 6, 1, 2, 1, 11, 19),
    _SnmpInTraps_Type()
)
snmpInTraps.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpInTraps.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpInTraps.setDescription("The total number of SNMP Trap PDUs which have been accepted and processed by the SNMP protocol entity.")
_SnmpOutTooBigs_Type = Counter32
_SnmpOutTooBigs_Object = MibScalar
snmpOutTooBigs = _SnmpOutTooBigs_Object(
    (1, 3, 6, 1, 2, 1, 11, 20),
    _SnmpOutTooBigs_Type()
)
snmpOutTooBigs.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutTooBigs.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutTooBigs.setDescription("The total number of SNMP PDUs which were generated by the SNMP protocol entity and for which the value of the error-status field was `tooBig.'")
_SnmpOutNoSuchNames_Type = Counter32
_SnmpOutNoSuchNames_Object = MibScalar
snmpOutNoSuchNames = _SnmpOutNoSuchNames_Object(
    (1, 3, 6, 1, 2, 1, 11, 21),
    _SnmpOutNoSuchNames_Type()
)
snmpOutNoSuchNames.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutNoSuchNames.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutNoSuchNames.setDescription("The total number of SNMP PDUs which were generated by the SNMP protocol entity and for which the value of the error-status was `noSuchName'.")
_SnmpOutBadValues_Type = Counter32
_SnmpOutBadValues_Object = MibScalar
snmpOutBadValues = _SnmpOutBadValues_Object(
    (1, 3, 6, 1, 2, 1, 11, 22),
    _SnmpOutBadValues_Type()
)
snmpOutBadValues.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutBadValues.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutBadValues.setDescription("The total number of SNMP PDUs which were generated by the SNMP protocol entity and for which the value of the error-status field was `badValue'.")
_SnmpOutGenErrs_Type = Counter32
_SnmpOutGenErrs_Object = MibScalar
snmpOutGenErrs = _SnmpOutGenErrs_Object(
    (1, 3, 6, 1, 2, 1, 11, 24),
    _SnmpOutGenErrs_Type()
)
snmpOutGenErrs.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutGenErrs.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutGenErrs.setDescription("The total number of SNMP PDUs which were generated by the SNMP protocol entity and for which the value of the error-status field was `genErr'.")
_SnmpOutGetRequests_Type = Counter32
_SnmpOutGetRequests_Object = MibScalar
snmpOutGetRequests = _SnmpOutGetRequests_Object(
    (1, 3, 6, 1, 2, 1, 11, 25),
    _SnmpOutGetRequests_Type()
)
snmpOutGetRequests.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutGetRequests.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutGetRequests.setDescription("The total number of SNMP Get-Request PDUs which have been generated by the SNMP protocol entity.")
_SnmpOutGetNexts_Type = Counter32
_SnmpOutGetNexts_Object = MibScalar
snmpOutGetNexts = _SnmpOutGetNexts_Object(
    (1, 3, 6, 1, 2, 1, 11, 26),
    _SnmpOutGetNexts_Type()
)
snmpOutGetNexts.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutGetNexts.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutGetNexts.setDescription("The total number of SNMP Get-Next PDUs which have been generated by the SNMP protocol entity.")
_SnmpOutSetRequests_Type = Counter32
_SnmpOutSetRequests_Object = MibScalar
snmpOutSetRequests = _SnmpOutSetRequests_Object(
    (1, 3, 6, 1, 2, 1, 11, 27),
    _SnmpOutSetRequests_Type()
)
snmpOutSetRequests.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutSetRequests.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutSetRequests.setDescription("The total number of SNMP Set-Request PDUs which have been generated by the SNMP protocol entity.")
_SnmpOutGetResponses_Type = Counter32
_SnmpOutGetResponses_Object = MibScalar
snmpOutGetResponses = _SnmpOutGetResponses_Object(
    (1, 3, 6, 1, 2, 1, 11, 28),
    _SnmpOutGetResponses_Type()
)
snmpOutGetResponses.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutGetResponses.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutGetResponses.setDescription("The total number of SNMP Get-Response PDUs which have been generated by the SNMP protocol entity.")
_SnmpOutTraps_Type = Counter32
_SnmpOutTraps_Object = MibScalar
snmpOutTraps = _SnmpOutTraps_Object(
    (1, 3, 6, 1, 2, 1, 11, 29),
    _SnmpOutTraps_Type()
)
snmpOutTraps.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpOutTraps.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpOutTraps.setDescription("The total number of SNMP Trap PDUs which have been generated by the SNMP protocol entity.")


class _SnmpEnableAuthenTraps_Type(Integer32):
    """Custom type snmpEnableAuthenTraps based on Integer32"""
    subtypeSpec = Integer32.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        SingleValueConstraint(
            *(1,
              2)
        )
    )
    namedValues = NamedValues(
        *(("enabled", 1),
          ("disabled", 2))
    )


_SnmpEnableAuthenTraps_Type.__name__ = "Integer32"
_SnmpEnableAuthenTraps_Object = MibScalar
snmpEnableAuthenTraps = _SnmpEnableAuthenTraps_Object(
    (1, 3, 6, 1, 2, 1, 11, 30),
    _SnmpEnableAuthenTraps_Type()
)
snmpEnableAuthenTraps.setMaxAccess("read-write")
if mibBuilder.loadTexts:
    snmpEnableAuthenTraps.setStatus("current")
if mibBuilder.loadTexts:
    snmpEnableAuthenTraps.setDescription("Indicates whether the SNMP entity is permitted to generate authenticationFailure traps. The value of this object overrides any configuration information; as such, it provides a means whereby all authenticationFailure traps may be disabled. Note that it is strongly recommended that this object be stored in non-volatile memory so that it remains constant across re-initializations of the network management system.")
_SnmpSilentDrops_Type = Counter32
_SnmpSilentDrops_Object = MibScalar
snmpSilentDrops = _SnmpSilentDrops_Object(
    (1, 3, 6, 1, 2, 1, 11, 31),
    _SnmpSilentDrops_Type()
)
snmpSilentDrops.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpSilentDrops.setStatus("current")
if mibBuilder.loadTexts:
    snmpSilentDrops.setDescription("The total number of Confirmed Class PDUs (such as GetRequest-PDUs, GetNextRequest-PDUs, GetBulkRequest-PDUs, SetRequest-PDUs, and InformRequest-PDUs) delivered to the SNMP entity which were silently dropped because the size of a reply containing an alternate Response Class PDU (such as a Response-PDU) with an empty variable-bindings field was greater than either a local constraint or the maximum message size associated with the originator of the request.")
_SnmpProxyDrops_Type = Counter32
_SnmpProxyDrops_Object = MibScalar
snmpProxyDrops = _SnmpProxyDrops_Object(
    (1, 3, 6, 1, 2, 1, 11, 32),
    _SnmpProxyDrops_Type()
)
snmpProxyDrops.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    snmpProxyDrops.setStatus("current")
if mibBuilder.loadTexts:
    snmpProxyDrops.setDescription("The total number of Confirmed Class PDUs (such as GetRequest-PDUs, GetNextRequest-PDUs, GetBulkRequest-PDUs, SetRequest-PDUs, and InformRequest-PDUs) delivered to the SNMP entity which were silently dropped because the transmission of the (possibly translated) message to a proxy target failed in a manner (other than a time-out) such that no Response Class PDU (such as a Response-PDU) could be returned.")
_SnmpMIBObjects_ObjectIdentity = ObjectIdentity
snmpMIBObjects = _SnmpMIBObjects_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 1, 1)
)
_SnmpTrap_ObjectIdentity = ObjectIdentity
snmpTrap = _SnmpTrap_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 1, 1, 4)
)
_SnmpTrapOID_Type = ObjectIdentifier
_SnmpTrapOID_Object = MibScalar
snmpTrapOID = _SnmpTrapOID_Object(
    (1, 3, 6, 1, 6, 3, 1, 1, 4, 1),
    _SnmpTrapOID_Type()
)
snmpTrapOID.setMaxAccess("accessible-for-notify")
if mibBuilder.loadTexts:
    snmpTrapOID.setStatus("current")
if mibBuilder.loadTexts:
    snmpTrapOID.setDescription("The authoritative identification of the notification currently being sent. This variable occurs as the second varbind in every SNMPv2-Trap-PDU and InformRequest-PDU.")
_SnmpTrapEnterprise_Type = ObjectIdentifier
_SnmpTrapEnterprise_Object = MibScalar
snmpTrapEnterprise = _SnmpTrapEnterprise_Object(
    (1, 3, 6, 1, 6, 3, 1, 1, 4, 3),
    _SnmpTrapEnterprise_Type()
)
snmpTrapEnterprise.setMaxAccess("accessible-for-notify")
if mibBuilder.loadTexts:
    snmpTrapEnterprise.setStatus("current")
if mibBuilder.loadTexts:
    snmpTrapEnterprise.setDescription("The authoritative identification of the enterprise associated with the trap currently being sent. When an SNMP proxy agent is mapping an RFC1157 Trap-PDU into a SNMPv2-Trap-PDU, this variable occurs as the last varbind.")
_SnmpTraps_ObjectIdentity = ObjectIdentity
snmpTraps = _SnmpTraps_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 1, 1, 5)
)
_SnmpSet_ObjectIdentity = ObjectIdentity
snmpSet = _SnmpSet_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 1, 1, 6)
)
_SnmpSetSerialNo_Type = TestAndIncr
_SnmpSetSerialNo_Object = MibScalar
snmpSetSerialNo = _SnmpSetSerialNo_Object(
    (1, 3, 6, 1, 6, 3, 1, 1, 6, 1),
    _SnmpSetSerialNo_Type()
)
snmpSetSerialNo.setMaxAccess("read-write")
if mibBuilder.loadTexts:
    snmpSetSerialNo.setStatus("current")
if mibBuilder.loadTexts:
    snmpSetSerialNo.setDescription("An advisory lock used to allow several cooperating command generator applications to coordinate their use of the SNMP set operation. This object is used for coarse-grain coordination. To achieve fine-grain coordination, one or more similar objects might be defined within each MIB group, as appropriate.")
_SnmpMIBConformance_ObjectIdentity = ObjectIdentity
snmpMIBConformance = _SnmpMIBConformance_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 1, 2)
)
_SnmpMIBCompliances_ObjectIdentity = ObjectIdentity
snmpMIBCompliances = _SnmpMIBCompliances_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 1, 2, 1)
)
_SnmpMIBGroups_ObjectIdentity = ObjectIdentity
snmpMIBGroups = _SnmpMIBGroups_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 1, 2, 2)
)

# Managed Objects groups

snmpSetGroup = ObjectGroup(
    (1, 3, 6, 1, 6, 3, 1, 2, 2, 5)
)
snmpSetGroup.setObjects(
    ("SNMPv2-MIB", "snmpSetSerialNo")
)
if mibBuilder.loadTexts:
    snmpSetGroup.setStatus("current")
if mibBuilder.loadTexts:
    snmpSetGroup.setDescription("A collection of objects which allow several cooperating command generator applications to coordinate their use of the set operation.")

systemGroup = ObjectGroup(
    (1, 3, 6, 1, 6, 3, 1, 2, 2, 6)
)
systemGroup.setObjects(
      *(("SNMPv2-MIB", "sysDescr"),
        ("SNMPv2-MIB", "sysObjectID"),
        ("SNMPv2-MIB", "sysUpTime"),
        ("SNMPv2-MIB", "sysContact"),
        ("SNMPv2-MIB", "sysName"),
        ("SNMPv2-MIB", "sysLocation"),
        ("SNMPv2-MIB", "sysServices"),
        ("SNMPv2-MIB", "sysORLastChange"),
        ("SNMPv2-MIB", "sysORID"),
        ("SNMPv2-MIB", "sysORUpTime"),
        ("SNMPv2-MIB", "sysORDescr"))
)
if mibBuilder.loadTexts:
    systemGroup.setStatus("current")
if mibBuilder.loadTexts:
    systemGroup.setDescription("The system group defines objects which are common to all managed systems.")

snmpGroup = ObjectGroup(
    (1, 3, 6, 1, 6, 3, 1, 2, 2, 8)
)
snmpGroup.setObjects(
      *(("SNMPv2-MIB", "snmpInPkts"),
        ("SNMPv2-MIB", "snmpInBadVersions"),
        ("SNMPv2-MIB", "snmpInASNParseErrs"),
        ("SNMPv2-MIB", "snmpSilentDrops"),
        ("SNMPv2-MIB", "snmpProxyDrops"),
        ("SNMPv2-MIB", "snmpEnableAuthenTraps"))
)
if mibBuilder.loadTexts:
    snmpGroup.setStatus("current")
if mibBuilder.loadTexts:
    snmpGroup.setDescription("A collection of objects providing basic instrumentation and control of an SNMP entity.")

snmpCommunityGroup = ObjectGroup(
    (1, 3, 6, 1, 6, 3, 1, 2, 2, 9)
)
snmpCommunityGroup.setObjects(
      *(("SNMPv2-MIB", "snmpInBadCommunityNames"),
        ("SNMPv2-MIB", "snmpInBadCommunityUses"))
)
if mibBuilder.loadTexts:
    snmpCommunityGroup.setStatus("current")
if mibBuilder.loadTexts:
    snmpCommunityGroup.setDescription("A collection of objects providing basic instrumentation of a SNMP entity which supports community-based authentication.")

snmpObsoleteGroup = ObjectGroup(
    (1, 3, 6, 1, 6, 3, 1, 2, 2, 10)
)
snmpObsoleteGroup.setObjects(
      *(("SNMPv2-MIB", "snmpOutPkts"),
        ("SNMPv2-MIB", "snmpInTooBigs"),
        ("SNMPv2-MIB", "snmpInNoSuchNames"),
        ("SNMPv2-MIB", "snmpInBadValues"),
        ("SNMPv2-MIB", "snmpInReadOnlys"),
        ("SNMPv2-MIB", "snmpInGenErrs"),
        ("SNMPv2-MIB", "snmpInTotalReqVars"),
        ("SNMPv2-MIB", "snmpInTotalSetVars"),
        ("SNMPv2-MIB", "snmpInGetRequests"),
        ("SNMPv2-MIB", "snmpInGetNexts"),
        ("SNMPv2-MIB", "snmpInSetRequests"),
        ("SNMPv2-MIB", "snmpInGetResponses"),
        ("SNMPv2-MIB", "snmpInTraps"),
        ("SNMPv2-MIB", "snmpOutTooBigs"),
        ("SNMPv2-MIB", "snmpOutNoSuchNames"),
        ("SNMPv2-MIB", "snmpOutBadValues"),
        ("SNMPv2-MIB", "snmpOutGenErrs"),
        ("SNMPv2-MIB", "snmpOutGetRequests"),
        ("SNMPv2-MIB", "snmpOutGetNexts"),
        ("SNMPv2-MIB", "snmpOutSetRequests"),
        ("SNMPv2-MIB", "snmpOutGetResponses"),
        ("SNMPv2-MIB", "snmpOutTraps"))
)
if mibBuilder.loadTexts:
    snmpObsoleteGroup.setStatus("obsolete")
if mibBuilder.loadTexts:
    snmpObsoleteGroup.setDescription("A collection of objects from RFC 1213 made obsolete by this MIB module.")

snmpNotificationGroup = ObjectGroup(
    (1, 3, 6, 1, 6, 3, 1, 2, 2, 12)
)
snmpNotificationGroup.setObjects(
      *(("SNMPv2-MIB", "snmpTrapOID"),
        ("SNMPv2-MIB", "snmpTrapEnterprise"))
)
if mibBuilder.loadTexts:
    snmpNotificationGroup.setStatus("current")
if mibBuilder.loadTexts:
    snmpNotificationGroup.setDescription("These objects are required for entities which support notification originator applications.")


# Notification objects

coldStart = NotificationType(
    (1, 3, 6, 1, 6, 3, 1, 1, 5, 1)
)
if mibBuilder.loadTexts:
    coldStart.setStatus(
        "current"
    )
if mibBuilder.loadTexts:
    coldStart.setDescription("A coldStart trap signifies that the SNMP entity, supporting a notification originator application, is reinitializing itself and that its configuration may have been altered.")

warmStart = NotificationType(
    (1, 3, 6, 1, 6, 3, 1, 1, 5, 2)
)
if mibBuilder.loadTexts:
    warmStart.setStatus(
        "current"
    )
if mibBuilder.loadTexts:
    warmStart.setDescription("A warmStart trap signifies that the SNMP entity, supporting a notification originator application, is reinitializing itself such that its configuration is unaltered.")

authenticationFailure = NotificationType(
    (1, 3, 6, 1, 6, 3, 1, 1, 5, 5)
)
if mibBuilder.loadTexts:
    authenticationFailure.setStatus(
        "current"
    )
if mibBuilder.loadTexts:
    authenticationFailure.setDescription("An authenticationFailure trap signifies that the SNMP entity has received a protocol message that is not properly authenticated. While all implementations of SNMP entities MAY be capable of generating this trap, the snmpEnableAuthenTraps object indicates whether this trap will be generated.")


# Notifications groups

snmpBasicNotificationsGroup = NotificationGroup(
    (1, 3, 6, 1, 6, 3, 1, 2, 2, 7)
)
snmpBasicNotificationsGroup.setObjects(
      *(("SNMPv2-MIB", "coldStart"),
        ("SNMPv2-MIB", "authenticationFailure"))
)
if mibBuilder.loadTexts:
    snmpBasicNotificationsGroup.setStatus(
        "current"
    )
if mibBuilder.loadTexts:
    snmpBasicNotificationsGroup.setDescription("The basic notifications implemented by an SNMP entity supporting command responder applications.")

snmpWarmStartNotificationGroup = NotificationGroup(
    (1, 3, 6, 1, 6, 3, 1, 2, 2, 11)
)
snmpWarmStartNotificationGroup.setObjects(
    ("SNMPv2-MIB", "warmStart")
)
if mibBuilder.loadTexts:
    snmpWarmStartNotificationGroup.setStatus(
        "current"
    )
if mibBuilder.loadTexts:
    snmpWarmStartNotificationGroup.setDescription("An additional notification for an SNMP entity supporting command responder applications, if it is able to reinitialize itself such that its configuration is unaltered.")


# Agent capabilities


# Module compliance

snmpBasicCompliance = ModuleCompliance(
    (1, 3, 6, 1, 6, 3, 1, 2, 1, 2)
)
snmpBasicCompliance.setObjects(
      *(("SNMPv2-MIB", "snmpGroup"),
        ("SNMPv2-MIB", "snmpSetGroup"),
        ("SNMPv2-MIB", "systemGroup"),
        ("SNMPv2-MIB", "snmpBasicNotificationsGroup"),
        ("SNMPv2-MIB", "snmpCommunityGroup"))
)
if mibBuilder.loadTexts:
    snmpBasicCompliance.setStatus(
        "deprecated"
    )
if mibBuilder.loadTexts:
    snmpBasicCompliance.setDescription("The compliance statement for SNMPv2 entities which implement the SNMPv2 MIB. This compliance statement is replaced by snmpBasicComplianceRev2.")

snmpBasicComplianceRev2 = ModuleCompliance(
    (1, 3, 6, 1, 6, 3, 1, 2, 1, 3)
)
snmpBasicComplianceRev2.setObjects(
      *(("SNMPv2-MIB", "snmpGroup"),
        ("SNMPv2-MIB", "snmpSetGroup"),
        ("SNMPv2-MIB", "systemGroup"),
        ("SNMPv2-MIB", "snmpBasicNotificationsGroup"),
        ("SNMPv2-MIB", "snmpCommunityGroup"),
        ("SNMPv2-MIB", "snmpWarmStartNotificationGroup"))
)
if mibBuilder.loadTexts:
    snmpBasicComplianceRev2.setStatus(
        "current"
    )
if mibBuilder.loadTexts:
    snmpBasicComplianceRev2.setDescription("The compliance statement for SNMP entities which implement this MIB module.")


# Export all MIB objects to the MIB builder

mibBuilder.export_symbols(
    "SNMPv2-MIB",
    **{"system": system,
       "sysDescr": sysDescr,
       "sysObjectID": sysObjectID,
       "sysUpTime": sysUpTime,
       "sysContact": sysContact,
       "sysName": sysName,
       "sysLocation": sysLocation,
       "sysServices": sysServices,
       "sysORLastChange": sysORLastChange,
       "sysORTable": sysORTable,
       "sysOREntry": sysOREntry,
       "sysORIndex": sysORIndex,
       "sysORID": sysORID,
       "sysORDescr": sysORDescr,
       "sysORUpTime": sysORUpTime,
       "snmp": snmp,
       "snmpInPkts": snmpInPkts,
       "snmpOutPkts": snmpOutPkts,
       "snmpInBadVersions": snmpInBadVersions,
       "snmpInBadCommunityNames": snmpInBadCommunityNames,
       "snmpInBadCommunityUses": snmpInBadCommunityUses,
       "snmpInASNParseErrs": snmpInASNParseErrs,
       "snmpInTooBigs": snmpInTooBigs,
       "snmpInNoSuchNames": snmpInNoSuchNames,
       "snmpInBadValues": snmpInBadValues,
       "snmpInReadOnlys": snmpInReadOnlys,
       "snmpInGenErrs": snmpInGenErrs,
       "snmpInTotalReqVars": snmpInTotalReqVars,
       "snmpInTotalSetVars": snmpInTotalSetVars,
       "snmpInGetRequests": snmpInGetRequests,
       "snmpInGetNexts": snmpInGetNexts,
       "snmpInSetRequests": snmpInSetRequests,
       "snmpInGetResponses": snmpInGetResponses,
       "snmpInTraps": snmpInTraps,
       "snmpOutTooBigs": snmpOutTooBigs,
       "snmpOutNoSuchNames": snmpOutNoSuchNames,
       "snmpOutBadValues": snmpOutBadValues,
       "snmpOutGenErrs": snmpOutGenErrs,
       "snmpOutGetRequests": snmpOutGetRequests,
       "snmpOutGetNexts": snmpOutGetNexts,
       "snmpOutSetRequests": snmpOutSetRequests,
       "snmpOutGetResponses": snmpOutGetResponses,
       "snmpOutTraps": snmpOutTraps,
       "snmpEnableAuthenTraps": snmpEnableAuthenTraps,
       "snmpSilentDrops": snmpSilentDrops,
       "snmpProxyDrops": snmpProxyDrops,
       "snmpMIB": snmpMIB,
       "snmpMIBObjects": snmpMIBObjects,
       "snmpTrap": snmpTrap,
       "snmpTrapOID": snmpTrapOID,
       "snmpTrapEnterprise": snmpTrapEnterprise,
       "snmpTraps": snmpTraps,
       "coldStart": coldStart,
       "warmStart": warmStart,
       "authenticationFailure": authenticationFailure,
       "snmpSet": snmpSet,
       "snmpSetSerialNo": snmpSetSerialNo,
       "snmpMIBConformance": snmpMIBConformance,
       "snmpMIBCompliances": snmpMIBCompliances,
       "snmpBasicCompliance": snmpBasicCompliance,
       "snmpBasicComplianceRev2": snmpBasicComplianceRev2,
       "snmpMIBGroups": snmpMIBGroups,
       "snmpSetGroup": snmpSetGroup,
       "systemGroup": systemGroup,
       "snmpBasicNotificationsGroup": snmpBasicNotificationsGroup,
       "snmpGroup": snmpGroup,
       "snmpCommunityGroup": snmpCommunityGroup,
       "snmpObsoleteGroup": snmpObsoleteGroup,
       "snmpWarmStartNotificationGroup": snmpWarmStartNotificationGroup,
       "snmpNotificationGroup": snmpNotificationGroup,
       "PYSNMP_MODULE_ID": snmpMIB}
)



"""
IPv6-to-IPv4 conversion
+++++++++++++++++++++++

Act as a local SNMPv1/v2c Agent listening on a UDP/IPv6 transport, relay
messages to distant SNMPv1/2c Agent over UDP/IPv4 transport:

* with local SNMPv2c community 'public'
* local Agent listening at [::1]:161
* remote SNMPv2c community 'public'
* remote Agent listening at 127.0.0.1:161

This script can be queried with the following Net-SNMP command:

| $ snmpget -v2c -c public udp6:[::1]:161 sysDescr.0

due to proxy, it is equivalent to

| $ snmpget -v2c -c public 127.0.0.1:161 sysDescr.0

Warning: for production operation you would need to modify this script
so that it will re-map possible duplicate request-ID values, coming in
initial request PDUs from different Managers, into unique values to
avoid sending duplicate request-IDs to Agents.

"""  #
from pysnmp.carrier.asyncio.dgram import udp, udp6
from pysnmp.entity import engine, config
from pysnmp.entity.rfc3413 import cmdrsp, cmdgen, context
from pysnmp.proto.api import v2c
from pysnmp import error

# Create SNMP engine with autogenernated engineID and pre-bound
# to socket transport dispatcher
snmpEngine = engine.SnmpEngine()

#
# Transport setup
#

# Agent section

# UDP over IPv6
config.add_transport(
    snmpEngine, udp6.DOMAIN_NAME, udp6.Udp6Transport().open_server_mode(("::1", 161))
)

# Manager section

# UDP over IPv4
config.add_transport(snmpEngine, udp.DOMAIN_NAME, udp.UdpTransport().open_client_mode())

#
# SNMPv1/2c setup (Agent role)
#

# SecurityName <-> CommunityName mapping
config.add_v1_system(snmpEngine, "1-my-area", "public")

#
# SNMPv1/v2c setup (Manager role)
#
# Here we configure securityName lexicographically lesser than '1-my-area'
# to let it match first in snmpCommunityTable on response processing.
#

config.add_v1_system(snmpEngine, "0-distant-area", "public", transportTag="remote")

#
# Transport target used by Manager
#

config.add_target_parameters(
    snmpEngine, "distant-agent-auth", "0-distant-area", "noAuthNoPriv", 1
)
config.add_target_address(
    snmpEngine,
    "distant-agent",
    udp.DOMAIN_NAME,
    ("127.0.0.1", 161),
    "distant-agent-auth",
    retryCount=0,
    tagList="remote",
)

# Default SNMP context
config.add_context(snmpEngine, "")


class CommandResponder(cmdrsp.CommandResponderBase):
    cmdGenMap = {
        v2c.GetRequestPDU.tagSet: cmdgen.GetCommandGenerator(),
        v2c.SetRequestPDU.tagSet: cmdgen.SetCommandGenerator(),
        v2c.GetNextRequestPDU.tagSet: cmdgen.NextCommandGeneratorSingleRun(),
        v2c.GetBulkRequestPDU.tagSet: cmdgen.BulkCommandGeneratorSingleRun(),
    }
    SUPPORTED_PDU_TYPES = cmdGenMap.keys()  # This app will handle these PDUs

    # SNMP request relay
    def handle_management_operation(
        self, snmpEngine, stateReference, contextName, PDU, acInfo
    ):
        cbCtx = stateReference, PDU
        contextEngineId = None  # address authoritative SNMP Engine
        try:
            self.cmdGenMap[PDU.tagSet].sendPdu(
                snmpEngine,
                "distant-agent",
                contextEngineId,
                contextName,
                PDU,
                self.handleResponsePdu,
                cbCtx,
            )
        except error.PySnmpError:
            self.handleResponsePdu(snmpEngine, stateReference, "error", None, cbCtx)

    # SNMP response relay
    # noinspection PyUnusedLocal
    def handleResponsePdu(
        self, snmpEngine, sendRequestHandle, errorIndication, PDU, cbCtx
    ):
        stateReference, reqPDU = cbCtx

        if errorIndication:
            PDU = v2c.apiPDU.get_response(reqPDU)
            PDU.set_error_status(PDU, 5)

        self.send_pdu(snmpEngine, stateReference, PDU)

        self.release_state_information(stateReference)


CommandResponder(snmpEngine, context.SnmpContext(snmpEngine))

snmpEngine.transport_dispatcher.job_started(1)  # this job would never finish

# Run I/O dispatcher which would receive queries and send responses
try:
    snmpEngine.open_dispatcher()
except:
    snmpEngine.close_dispatcher()
    raise
